use std::fs;

use anyhow::{bail, Context};
use serde::Deserialize;

#[derive(Deserialize, Clone)]
pub struct FelinetConfig {
    pub address: String,
    pub callsign: String,
    pub ssid: u8,
}

#[derive(Deserialize, Clone)]
pub struct Config {
    pub felinet: Option<FelinetConfig>,
}

impl Config {
    pub fn load() -> anyhow::Result<Self> {
        let file_contents = read_conf_file()?;
        let config: Self = toml::from_str(&file_contents).context("parsing config file")?;

        if config
            .felinet
            .as_ref()
            .map(|f| f.callsign == "CHANGEME")
            .unwrap_or(false)
        {
            bail!("Callsign is not configured in config file");
        }

        Ok(config)
    }
}

fn read_conf_file() -> anyhow::Result<String> {
    let filenames = ["config.toml", "/etc/cats_sdr_igate/config.toml"];
    for filename in filenames.iter() {
        if let Ok(x) = fs::read_to_string(filename) {
            return Ok(x);
        }
    }

    bail!("Could not open config file. Should either be `config.toml` in the current directory, or in /etc/cats_sdr_igate/")
}
