use ham_cats::packet::Packet;

use crate::MAX_PACKET_LEN;

use super::{
    packet::{PacketDecoder, Status},
    Bit,
};

const SYNC_WORD: u32 = 0xABCDEF12;
const MAX_BIT_FLIPS: u32 = 3;

pub struct Demod<I: Iterator<Item = f32>> {
    rolling_sync: u32,
    iter: I,
    decoders: Vec<PacketDecoder>,
}

impl<I: Iterator<Item = f32>> Demod<I> {
    pub fn new(iter: I) -> Self {
        Self {
            rolling_sync: 0,
            iter,
            decoders: vec![],
        }
    }

    pub fn demod<F: Fn(Packet<MAX_PACKET_LEN>)>(mut self, handler: F) -> Option<()> {
        loop {
            let x = self.iter.next()?;
            let bit = u32::from(x.bit());

            for d in &mut self.decoders {
                d.push(x);
            }

            self.rolling_sync <<= 1;
            self.rolling_sync |= bit;

            let mut to_remove = vec![];
            for (i, d) in self.decoders.iter_mut().enumerate() {
                match d.decode(&handler) {
                    Status::Some => {
                        to_remove.push(i);
                        break;
                    }
                    Status::NotYet => {}
                    Status::Never => to_remove.push(i),
                }
            }

            for i in to_remove {
                self.decoders.swap_remove(i);
            }

            if self.sync_matches() {
                self.decoders.push(PacketDecoder::new());
            }
        }
    }

    fn sync_matches(&self) -> bool {
        let delta = self.rolling_sync ^ SYNC_WORD;

        delta.count_ones() < MAX_BIT_FLIPS
    }
}
